---
layout: base
---

<!-- animated sidebar (can be revealed in mobile view) -->
<div id="sidebar-retractable" class="ui vertical inverted sidebar menu">
  <!-- #sidebar-collection-indices is re-parented here for mobile view -->
</div>

<!-- pushing container (will be pushed right when retractable sidebar opens) -->
<div class="pusher">

<div class="ui left attached internal rail">
<!-- static sidebar (visible in non-mobile view) -->
<div id="sidebar-fixed" class="ui vertical inverted menu">
<!--<a target="_blank" href="{{ site.baseurl }}/"><img style="width:220px; padding-top:45px; padding-left:10px" src="{{ site.baseurl }}/assets/img/contributor-ci.png"/></a>-->
<div id="sidebar-collection-indices">
{% include elements/indices.html %}
</div>
</div>
</div>

<!-- main contents -->
<div id="page-container">
<div class="ui left aligned fluid container">
<div id="page-content" class="ui basic segment">
{{ content }}
</div>
</div>
</div>

</div>

<!-- help modal -->
<div id="site-help" class="ui small modal">
{% include elements/help.html %}
</div>

<script>
{% include scripts/indices.js %}
{% include scripts/search.js %}
{% include scripts/help.js %}

{% comment %} provide user extension / override point {% endcomment %}
{% include scripts/custom.js %}


</script>
