query ($ownName: String!, $repoName: String!) {
  repository(owner: $ownName, name: $repoName) {
    nameWithOwner
    name
    languages(first: 1, orderBy: {field: SIZE, direction: DESC}) {
      nodes {
        name
        color
      }
    }
    owner {
      login
      ... on Organization {
        isVerified
      }
    }
  }
}
# {"ownName": "LLNL", "repoName": "llnl.github.io"}
